@@ -15,8 +15,6 @@
 #   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #   See the License for the specific language governing permissions and
 #   limitations under the License.
-#
-#   Version v0.1.1
 import re
 import itertools
 import base64
@@ -924,20 +922,19 @@ class RootNode(BXmlNode):
             sub_decl.append((size, type_))
             ofs += 4
         for (size, type_) in sub_decl:
-            value = None
             #[0] = parse_null_type_node,
             if type_ == 0x0:
                 value = None
                 sub_def.append(value)
             #[1] = parse_wstring_type_node,
             elif type_ == 0x1:
-                s = self.unpack_wstring(ofs, size / 2)
+                s = self.unpack_wstring(ofs, size / 2).rstrip("\x00")
                 value = s.replace("<", "&gt;").replace(">", "&lt;")
                 sub_def.append(value)
             #[2] = parse_string_type_node,
             elif type_ == 0x2:
                 s = self.unpack_string(ofs, size)
-                value = s.decode("utf8")
+                value = s.decode("utf8").rstrip("\x00")
                 value = value.replace("<", "&gt;")
                 value = value.replace(">", "&lt;")
                 sub_def.append(value)
@@ -998,8 +995,8 @@ class RootNode(BXmlNode):
             elif type_ == 0x13:
                 version = self.unpack_byte(ofs)
                 num_elements = self.unpack_byte(ofs + 1)
-                id_high = self.unpack_dword_be(ofs)
-                id_low = self.unpack_word_be(ofs)
+                id_high = self.unpack_dword_be(ofs + 2)
+                id_low = self.unpack_word_be(ofs + 6)
                 value = "S-%d-%d" % (version, (id_high << 16) ^ id_low)
                 for i in xrange(num_elements):
                     val = self.unpack_dword(ofs + 8 + (4 * i))
@@ -1081,7 +1078,6 @@ class RootNode(BXmlNode):
 
     @memoize
     def length(self):
-        ret = 0
         ofs = self.tag_and_children_length()
         sub_count = self.unpack_dword(ofs)
         ofs += 4
